@@ -3,14 +3,15 @@
 import ast
 import builtins
 import copy
+import functools
 import importlib
 import inspect
 import io
 import logging
 import os.path
-import platform
 import pkgutil
 import pydoc
+import re
 import signal
 import site
 import subprocess
@@ -21,11 +22,11 @@ import types
 import warnings
 from collections import namedtuple
 from importlib.machinery import PathFinder, SourceFileLoader
+from threading import Thread
 
 import __main__  # @UnresolvedImport
 import _ast
 import thonny
-from thonny import ast_utils
 from thonny.common import (
     BackendEvent,
     DebuggerCommand,
@@ -44,7 +45,8 @@ from thonny.common import (
     range_contains_smaller,
     range_contains_smaller_or_equal,
     serialize_message,
-)
+    get_exe_dirs, get_augmented_system_path, update_system_path)
+import queue
 
 BEFORE_STATEMENT_MARKER = "_thonny_hidden_before_stmt"
 BEFORE_EXPRESSION_MARKER = "_thonny_hidden_before_expr"
@@ -83,6 +85,8 @@ class VM:
         self._command_handlers = {}
         self._object_info_tweakers = []
         self._import_handlers = {}
+        self._input_queue = queue.Queue()
+        self._ast_postprocessors = []
         self._main_dir = os.path.dirname(sys.modules["thonny"].__file__)
         self._heap = (
             {}
@@ -90,7 +94,6 @@ class VM:
         self._source_info_by_frame = {}
         site.sethelper()  # otherwise help function is not available
         pydoc.pager = pydoc.plainpager  # otherwise help command plays tricks
-        self._tweak_system_path()
         self._install_fake_streams()
         self._current_executor = None
         self._io_level = 0
@@ -103,31 +106,20 @@ class VM:
         # clean up path
         sys.path = [d for d in sys.path if d != ""]
 
-        # script mode
-        if len(sys.argv) > 1:
-            special_names_to_remove = set()
-            sys.argv[:] = sys.argv[1:]  # shift argv[1] to position of script name
-            sys.path.insert(
-                0, os.path.abspath(os.path.dirname(sys.argv[0]))
-            )  # add program's dir
-            __main__.__dict__["__file__"] = sys.argv[0]
-            # TODO: inspect.getdoc
-
-        # shell mode
-        else:
-            special_names_to_remove = {"__file__", "__cached__"}
-            sys.argv[:] = [""]  # empty "script name"
-            sys.path.insert(0, "")  # current dir
-
+        # start in shell mode
+        sys.argv[:] = [""]  # empty "script name"
+        sys.path.insert(0, "")  # current dir
+        
         # clean __main__ global scope
         for key in list(__main__.__dict__.keys()):
-            if not key.startswith("__") or key in special_names_to_remove:
+            if not key.startswith("__") or key in {"__file__", "__cached__"}:
                 del __main__.__dict__[key]
 
         # unset __doc__, then exec dares to write doc of the script there
         __main__.__doc__ = None
-
-        self._load_shared_modules(init_msg["frontend_sys_path"])
+        
+        self._frontend_sys_path = init_msg["frontend_sys_path"]
+        self._load_shared_modules()
         self._load_plugins()
 
         self.send_message(
@@ -138,11 +130,12 @@ class VM:
                 argv=sys.argv,
                 path=sys.path,
                 usersitepackages=site.getusersitepackages()
-                if site.ENABLE_USER_SITE
-                else None,
+                    if site.ENABLE_USER_SITE
+                    else None,
                 prefix=sys.prefix,
                 welcome_text="Python " + _get_python_version_string(),
                 executable=sys.executable,
+                exe_dirs=get_exe_dirs(),
                 in_venv=(
                     hasattr(sys, "base_prefix")
                     and sys.base_prefix != sys.prefix
@@ -161,8 +154,14 @@ class VM:
             while True:
                 try:
                     cmd = self._fetch_command()
-                    self._source_info_by_frame = {}
-                    self.handle_command(cmd)
+                    if isinstance(cmd, InputSubmission):
+                        self._input_queue.put(cmd)
+                    elif isinstance(cmd, ToplevelCommand):
+                        self._source_info_by_frame = {}
+                        self._input_queue = queue.Queue()
+                        self.handle_command(cmd)
+                    else:
+                        self.handle_command(cmd)
                 except KeyboardInterrupt:
                     logger.exception("Interrupt in mainloop")
                     # Interrupt must always result in waiting_toplevel_command state
@@ -181,7 +180,7 @@ class VM:
         or a BackendResponse
         """
         self._command_handlers[command_name] = handler
-
+    
     def add_object_info_tweaker(self, tweaker):
         """Tweaker should be 2-argument function taking value and export record"""
         self._object_info_tweakers.append(tweaker)
@@ -190,7 +189,10 @@ class VM:
         if module_name not in self._import_handlers:
             self._import_handlers[module_name] = []
         self._import_handlers[module_name].append(handler)
-
+    
+    def add_ast_postprocessor(self, func):
+        self._ast_postprocessors.append(func)
+    
     def get_main_module(self):
         return __main__
 
@@ -265,7 +267,17 @@ class VM:
             value = repr(value)
         ini.set(section, subname, value)
         self.save_settings()
-
+    
+    def switch_env_to_script_mode(self, cmd):
+        if "" in sys.path:
+            sys.path.remove("") # current directory
+            
+        filename = cmd.args[0]
+        if os.path.isfile(filename):
+            sys.path.insert(0, os.path.abspath(os.path.dirname(filename))) 
+            __main__.__dict__["__file__"] = filename
+        
+    
     def _parse_option_name(self, name):
         if "." in name:
             return name.split(".", 1)
@@ -288,31 +300,7 @@ class VM:
         with open(_CONFIG_FILENAME, "w") as fp:
             self._ini.write(fp)
     
-    def _tweak_system_path(self):
-        """"allow easy calling of Python scripts """
-        
-        def prepend_if_missing(script_dir):
-            sys_path = os.environ.get("PATH", "")
-            if script_dir not in sys_path.split(os.pathsep):
-                os.environ["PATH"] = script_dir + os.pathsep + sys_path
-                
-        try:
-            if platform.system() == "Windows":
-                base_scripts = sys.prefix + "\\Scripts"
-            else:
-                base_scripts = sys.prefix + "/bin"
-            prepend_if_missing(base_scripts)
-            
-            if platform.system() == "Windows":
-                user_scripts = site.USER_SITE.replace("site-packages", "Scripts")
-            else:
-                user_scripts = site.USER_BASE + "/bin"
-            prepend_if_missing(user_scripts)
-            
-        except Exception:
-            logging.getLogger("thonny").exception("Couldn't tweak system path")
-            pass
-
+    
     def _custom_import(self, *args, **kw):
         module = self._original_import(*args, **kw)
         
@@ -335,9 +323,21 @@ class VM:
 
         return module
 
-    def _load_shared_modules(self, frontend_sys_path):
-        for name in ["parso", "jedi", "thonnycontrib"]:
-            load_module_from_alternative_path(name, frontend_sys_path)
+    def _load_shared_modules(self):
+        self.load_modules_with_frontend_path(["parso", "jedi", "thonnycontrib", "six", "asttokens"])
+
+    def load_modules_with_frontend_path(self, names):
+        from importlib import import_module
+        original_sys_path = sys.path
+        try:
+            sys.path = sys.path + self._frontend_sys_path
+            for name in names:
+                try:
+                    import_module(name)
+                except ImportError:
+                    pass
+        finally:
+            sys.path = original_sys_path
 
     def _load_plugins(self):
         # built-in plugins
@@ -396,15 +396,18 @@ class VM:
             raise UserError("cd takes one parameter")
 
     def _cmd_Run(self, cmd):
+        self.switch_env_to_script_mode(cmd)
         return self._execute_file(cmd, SimpleRunner)
 
     def _cmd_run(self, cmd):
         return self._execute_file(cmd, SimpleRunner)
 
     def _cmd_FastDebug(self, cmd):
+        self.switch_env_to_script_mode(cmd)
         return self._execute_file(cmd, FastTracer)
 
     def _cmd_Debug(self, cmd):
+        self.switch_env_to_script_mode(cmd)
         return self._execute_file(cmd, NiceTracer)
 
     def _cmd_debug(self, cmd):
@@ -447,10 +450,12 @@ class VM:
         return ToplevelResponse(command_name="execute_source", **result_attributes)
 
     def _cmd_execute_system_command(self, cmd):
-        # TODO: how to publish stdout as it arrives?
         env = dict(os.environ).copy()
         encoding = "utf-8"
         env["PYTHONIOENCODING"] = encoding
+        # Make sure this python interpreter and its scripts are available
+        # in PATH 
+        update_system_path(env, get_augmented_system_path(get_exe_dirs()))
         popen_kw = dict(
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
@@ -460,15 +465,33 @@ class VM:
         )
         
         if sys.version_info >= (3,6):
-            popen_kw["errors"] = "backslashreplace"
+            popen_kw["errors"] = "replace"
             popen_kw["encoding"] = encoding
         
         assert cmd.cmd_line.startswith("!")
         cmd_line = cmd.cmd_line[1:]
         proc = subprocess.Popen(cmd_line, **popen_kw)
-        out, err = proc.communicate(input="")
-        print(out, end="")
-        print(err, file=sys.stderr, end="")
+        
+        def copy_stream(source, target):
+            while True:
+                c = source.readline()
+                if c == "":
+                    break
+                else:
+                    target.write(c)
+        
+        copy_out = Thread(target=lambda: copy_stream(proc.stdout, sys.stdout), daemon=True)
+        copy_err = Thread(target=lambda: copy_stream(proc.stderr, sys.stderr), daemon=True)
+        
+        copy_out.start()
+        copy_err.start()
+        try:
+            proc.wait()
+        except KeyboardInterrupt as e:
+            print(str(e), file=sys.stderr)
+        
+        copy_out.join()
+        copy_err.join()
 
     def _cmd_process_gui_events(self, cmd):
         # advance the event loop
@@ -802,7 +825,7 @@ class VM:
         # TODO: are they?
 
         if len(cmd.args) >= 1:
-            sys.argv = cmd.args  # TODO: duplicate????????????????????
+            sys.argv = cmd.args  
             filename = cmd.args[0]
             if os.path.isabs(filename):
                 full_filename = filename
@@ -813,19 +836,21 @@ class VM:
                 source = fp.read()
 
             result_attributes = self._execute_source(
-                source, full_filename, "exec", executor_class, cmd
+                source, full_filename, "exec", executor_class, cmd,
+                self._ast_postprocessors
             )
             result_attributes["filename"] = full_filename
             return ToplevelResponse(command_name=cmd.name, **result_attributes)
         else:
             raise UserError("Command '%s' takes at least one argument" % cmd.name)
 
-    def _execute_source(self, source, filename, execution_mode, executor_class, cmd):
+    def _execute_source(self, source, filename, execution_mode, executor_class,
+                        cmd, ast_postprocessors=[]):
         self._current_executor = executor_class(self, cmd)
 
         try:
             return self._current_executor.execute_source(
-                source, filename, execution_mode
+                source, filename, execution_mode, ast_postprocessors
             )
         finally:
             self._current_executor = None
@@ -873,7 +898,12 @@ class VM:
 
     def export_value(self, value, max_repr_length=5000):
         self._heap[id(value)] = value
-        rep = repr(value)
+        try:
+            rep = repr(value)
+        except Exception:
+            # See https://bitbucket.org/plas/thonny/issues/584/problem-with-thonnys-back-end-obj-no
+            rep = "??? <repr error>"
+            
         if len(rep) > max_repr_length:
             rep = rep[:max_repr_length] + "…"
 
@@ -905,7 +935,7 @@ class VM:
     def is_doing_io(self):
         return self._io_level > 0
 
-    def _export_stack(self, newest_frame):
+    def _export_stack(self, newest_frame, skip_checker=None):
         result = []
 
         system_frame = newest_frame
@@ -913,39 +943,40 @@ class VM:
         while system_frame is not None:
             module_name = system_frame.f_globals["__name__"]
             code_name = system_frame.f_code.co_name
-
-            source, firstlineno, in_library = self._get_frame_source_info(system_frame)
-
-            result.insert(
-                0,
-                FrameInfo(
-                    # TODO: can this id be reused by a later frame?
-                    # Need to store the refernce to avoid GC?
-                    # I guess it is not required, as id will be required
-                    # only for stacktrace inspection, and sys.last_exception
-                    # will have the reference anyway
-                    # (NiceTracer has its own reference keeping)
-                    id=id(system_frame),
-                    filename=system_frame.f_code.co_filename,
-                    module_name=module_name,
-                    code_name=code_name,
-                    locals=self.export_variables(system_frame.f_locals),
-                    globals=self.export_variables(system_frame.f_globals),
-                    freevars=system_frame.f_code.co_freevars,
-                    source=source,
-                    lineno=system_frame.f_lineno,
-                    firstlineno=firstlineno,
-                    in_library=in_library,
-                    event="line",
-                    focus=TextRange(
-                        system_frame.f_lineno, 0, system_frame.f_lineno + 1, 0
+            
+            if not skip_checker or not skip_checker(system_frame):
+                source, firstlineno, in_library = self._get_frame_source_info(system_frame)
+    
+                result.insert(
+                    0,
+                    FrameInfo(
+                        # TODO: can this id be reused by a later frame?
+                        # Need to store the refernce to avoid GC?
+                        # I guess it is not required, as id will be required
+                        # only for stacktrace inspection, and sys.last_exception
+                        # will have the reference anyway
+                        # (NiceTracer has its own reference keeping)
+                        id=id(system_frame),
+                        filename=system_frame.f_code.co_filename,
+                        module_name=module_name,
+                        code_name=code_name,
+                        locals=self.export_variables(system_frame.f_locals),
+                        globals=self.export_variables(system_frame.f_globals),
+                        freevars=system_frame.f_code.co_freevars,
+                        source=source,
+                        lineno=system_frame.f_lineno,
+                        firstlineno=firstlineno,
+                        in_library=in_library,
+                        event="line",
+                        focus=TextRange(
+                            system_frame.f_lineno, 0, system_frame.f_lineno + 1, 0
+                        ),
+                        node_tags=None,
+                        current_statement=None,
+                        current_evaluations=None,
+                        current_root_expression=None,
                     ),
-                    node_tags=None,
-                    current_statement=None,
-                    current_evaluations=None,
-                    current_root_expression=None,
-                ),
-            )
+                )
 
             if module_name == "__main__" and code_name == "<module>":
                 # this was last frame relevant to the user
@@ -1075,6 +1106,13 @@ class VM:
 
     class FakeInputStream(FakeStream):
         def _generic_read(self, method, limit=-1):
+            # is there some queued input?
+            if not self._vm._input_queue.empty():
+                cmd = self._vm._input_queue.get()
+                self._processed_symbol_count += len(cmd.data)
+                return cmd.data
+            
+            # new input needs to be requested
             try:
                 self._vm._enter_io_function()
                 self._vm.send_message(
@@ -1103,6 +1141,41 @@ class VM:
 
         def readlines(self, limit=-1):
             return self._generic_read("readlines", limit)
+        
+        def __next__(self):
+            return self.readline()
+        
+        def __iter__(self):
+            return self
+
+
+def prepare_hooks(method):
+    @functools.wraps(method)
+    def wrapper(self, *args, **kwargs):
+        try:
+            sys.meta_path.insert(0, self)
+            self._vm._install_custom_import()
+            return method(self, *args, **kwargs)
+        finally:
+            del sys.meta_path[0]
+            if hasattr(self._vm, "_original_import"):
+                self._vm._restore_original_import()
+
+    return wrapper
+
+
+def return_execution_result(method):
+    @functools.wraps(method)
+    def wrapper(self, *args, **kwargs):
+        try:
+            result = method(self, *args, **kwargs)
+            if result is not None:
+                return result
+            return {"context_info": "after normal execution"}
+        except Exception:
+            return {"user_exception": self._vm._prepare_user_exception()}
+
+    return wrapper
 
 
 class Executor:
@@ -1111,7 +1184,7 @@ class Executor:
         self._original_cmd = original_cmd
         self._main_module_path = None
 
-    def execute_source(self, source, filename, mode):
+    def execute_source(self, source, filename, mode, ast_postprocessors):
         if isinstance(source, str):
             # TODO: simplify this or make sure encoding is correct
             source = source.encode("utf-8")
@@ -1120,27 +1193,30 @@ class Executor:
             self._main_module_path = filename
 
         global_vars = __main__.__dict__
+        statements = expression = None
 
         try:
             if mode == "exec+eval":
+                assert not ast_postprocessors 
                 # Useful in shell to get last expression value in multi-statement block
                 root = self._prepare_ast(source, filename, "exec")
                 statements = compile(ast.Module(body=root.body[:-1]), filename, "exec")
                 expression = compile(
                     ast.Expression(root.body[-1].value), filename, "eval"
                 )
-                return self._prepare_hooks_and_execute(
-                    statements, expression, global_vars
-                )
             else:
                 root = self._prepare_ast(source, filename, mode)
-                bytecode = compile(root, filename, mode)
                 if mode == "eval":
-                    return self._prepare_hooks_and_execute(None, bytecode, global_vars)
+                    assert not ast_postprocessors 
+                    expression = compile(root, filename, mode)
                 elif mode == "exec":
-                    return self._prepare_hooks_and_execute(bytecode, None, global_vars)
+                    for func in ast_postprocessors:
+                        func(root)
+                    statements = compile(root, filename, mode)
                 else:
                     raise ValueError("Unknown mode")
+
+            return self._execute_prepared_user_code(statements, expression, global_vars)
         except SyntaxError:
             return {"user_exception": self._vm._prepare_user_exception()}
         except SystemExit:
@@ -1149,28 +1225,16 @@ class Executor:
             _report_internal_error()
             return {}
 
-    def _prepare_hooks_and_execute(self, statements, expression, global_vars):
-        try:
-            sys.meta_path.insert(0, self)
-            self._vm._install_custom_import()
-            return self._execute_prepared_user_code(statements, expression, global_vars)
-        finally:
-            del sys.meta_path[0]
-            self._vm._restore_original_import()
-
+    @return_execution_result
+    @prepare_hooks
     def _execute_prepared_user_code(self, statements, expression, global_vars):
-        try:
-            if statements:
-                exec(statements, global_vars)
-            if expression:
-                value = eval(expression, global_vars)
-                if value is not None:
-                    builtins._ = value
-                return {"value_info": self._vm.export_value(value)}
-
-            return {"context_info": "after normal execution"}
-        except Exception:
-            return {"user_exception": self._vm._prepare_user_exception()}
+        if statements:
+            exec(statements, global_vars)
+        if expression:
+            value = eval(expression, global_vars)
+            if value is not None:
+                builtins._ = value
+            return {"value_info": self._vm.export_value(value)}
 
     def find_spec(self, fullname, path=None, target=None):
         """override in subclass for custom-loading user modules"""
@@ -1246,6 +1310,7 @@ class Tracer(Executor):
             and code.co_flags & inspect.CO_ASYNC_GENERATOR  # @UndefinedVariable
             or "importlib._bootstrap" in code.co_filename
             or self._vm.is_doing_io()
+            or path_startswith(code.co_filename, self._thonny_src_dir)
         )
 
     def _is_interesting_module_file(self, path):
@@ -1295,6 +1360,7 @@ class Tracer(Executor):
                 "type_name": None,
                 "lines_with_frame_info": None,
                 "affected_frame_ids": set(),
+                "is_fresh": False, 
             }
         else:
             return {
@@ -1303,6 +1369,7 @@ class Tracer(Executor):
                 "type_name": exc[0].__name__,
                 "lines_with_frame_info": format_exception_with_frame_info(*exc),
                 "affected_frame_ids": exc[1]._affected_frame_ids_,
+                "is_fresh": exc == self._fresh_exception, 
             }
 
     def _get_breakpoints_with_cursor_position(self, cmd):
@@ -1377,7 +1444,7 @@ class FastTracer(Tracer):
 
     def _report_current_state(self, frame):
         msg = DebuggerResponse(
-            stack=self._vm._export_stack(frame),
+            stack=self._vm._export_stack(frame, self._should_skip_frame),
             in_present=True,
             io_symbol_count=None,
             exception_info=self._export_exception_info(),
@@ -1418,12 +1485,6 @@ class FastTracer(Tracer):
         filename = frame.f_code.co_filename
         return filename in breakpoints and frame.f_lineno in breakpoints[filename]
 
-    def _should_skip_frame(self, frame):
-        code = frame.f_code
-        return super()._should_skip_frame(frame) or code.co_filename.startswith(
-            self._thonny_src_dir
-        )
-
     def _frame_is_alive(self, frame_id):
         return frame_id in self._alive_frame_ids
 
@@ -1461,6 +1522,9 @@ class NiceTracer(Tracer):
                 setattr(builtins, name, getattr(self, name))
 
     def _prepare_ast(self, source, filename, mode):
+        # ast_utils need to be imported after asttokens 
+        # is (custom-)imported 
+        from thonny import ast_utils
         root = ast.parse(source, filename, mode)
 
         ast_utils.mark_text_ranges(root, source)
@@ -1475,12 +1539,11 @@ class NiceTracer(Tracer):
     def _should_skip_frame(self, frame):
         code = frame.f_code
         return (
-            code.co_name
-            not in self.marker_function_names  # never skip marker functions
+            # never skip marker functions
+            code.co_name not in self.marker_function_names  
             and (
                 super()._should_skip_frame(frame)
                 or code.co_filename not in self._instrumented_files
-                or path_startswith(code.co_filename, self._thonny_src_dir)
             )
         )
 
@@ -1502,6 +1565,14 @@ class NiceTracer(Tracer):
         return self._current_state_index < len(self._saved_states) - 1
 
     def _trace(self, frame, event, arg):
+        try:
+            return self._trace_and_catch(frame, event, arg)
+        except BaseException:
+            traceback.print_exc()
+            sys.settrace(None)
+            return None
+            
+    def _trace_and_catch(self, frame, event, arg):
         """
         1) Detects marker calls and responds to client queries in these spots
         2) Maintains a customized view of stack
@@ -1561,7 +1632,6 @@ class NiceTracer(Tracer):
             pseudo_event = last_custom_frame.event.replace("before_", "after_").replace(
                 "_again", ""
             )
-
             self._handle_progress_event(frame, pseudo_event, {}, last_custom_frame.node)
 
         elif event == "return":
@@ -1646,6 +1716,7 @@ class NiceTracer(Tracer):
             prev_state is not None
             and id(prev_state_frame.system_frame) == id(frame)
             and prev_state["exception_value"] is self._get_current_exception()[1]
+            and prev_state["fresh_exception_id"] == id(self._fresh_exception)
             and ("before" in event or "skipexport" in node.tags)
         ):
 
@@ -1677,6 +1748,7 @@ class NiceTracer(Tracer):
                 + sys.stderr._processed_symbol_count
             ),
             "exception_value": self._get_current_exception()[1],
+            "fresh_exception_id": id(self._fresh_exception),
             "exception_info": exception_info,
         }
 
@@ -2010,7 +2082,10 @@ class NiceTracer(Tracer):
 
     def _tag_nodes(self, root):
         """Marks interesting properties of AST nodes"""
-
+        # ast_utils need to be imported after asttokens 
+        # is (custom-)imported 
+        from thonny import ast_utils
+        
         def add_tag(node, tag):
             if not hasattr(node, "tags"):
                 node.tags = set()
@@ -2196,7 +2271,11 @@ class NiceTracer(Tracer):
     def _insert_statement_markers(self, root):
         # find lists of statements and insert before/after markers for each statement
         for name, value in ast.iter_fields(root):
-            if isinstance(value, ast.AST):
+            if isinstance(root, ast.Try) and name == "handlers":
+                # contains statements but is not statement itself
+                for handler in value:
+                    self._insert_statement_markers(handler)
+            elif isinstance(value, ast.AST):
                 self._insert_statement_markers(value)
             elif isinstance(value, list):
                 if len(value) > 0:
@@ -2417,18 +2496,30 @@ def _get_python_version_string(add_word_size=False):
 def _fetch_frame_source_info(frame):
     if frame.f_code.co_filename is None or not os.path.exists(frame.f_code.co_filename):
         return None, None, True
-
+    
+    is_libra = _is_library_file(frame.f_code.co_filename)
     if frame.f_code.co_name == "<module>":
         # inspect.getsource and getsourcelines don't help here
         with tokenize.open(frame.f_code.co_filename) as fp:
-            return fp.read(), 1, _is_library_file(frame.f_code.co_filename)
+            return fp.read(), 1, is_libra
     else:
+        # function or class
         try:
-            return (
-                inspect.getsource(frame),
-                frame.f_code.co_firstlineno,
-                _is_library_file(frame.f_code.co_filename),
-            )
+            source = inspect.getsource(frame.f_code)
+            
+            # inspect.getsource is not reliable, see eg:
+            # https://bugs.python.org/issue35101
+            # If the code name is not present as definition
+            # in the beginning of the source,
+            # then play safe and return the whole script 
+            first_line = source.splitlines()[0]
+            if re.search(r"\b(class|def)\b\s+\b%s\b" % frame.f_code.co_name,
+                         first_line) is None:
+                with tokenize.open(frame.f_code.co_filename) as fp:
+                    return fp.read(), 1, is_libra
+                
+            else:
+                return source, frame.f_code.co_firstlineno, is_libra 
         except OSError:
             logger.exception("Problem getting source")
             return None, None, True
@@ -2531,20 +2622,6 @@ def format_exception_with_frame_info(
     return list(items)
 
 
-def load_module_from_alternative_path(module_name, path, force=False):
-    spec = PathFinder.find_spec(module_name, path)
-
-    if spec is None and not force:
-        return
-
-    from importlib.util import module_from_spec
-
-    module = module_from_spec(spec)
-    sys.modules[module_name] = module
-    if spec.loader is not None:
-        spec.loader.exec_module(module)
-
-
 def in_debug_mode():
     return os.environ.get("THONNY_DEBUG", False) in [1, "1", True, "True", "true"]
 
