@@ -157,6 +157,8 @@ class Workbench(tk.Tk):
         self._init_regular_mode_link()
 
         self._show_views()
+        # Make sure ShellView is loaded
+        self.get_view("ShellView")
 
         self._init_commands()
         self._init_icon()
@@ -202,6 +204,7 @@ class Workbench(tk.Tk):
         self.set_default("general.debug_mode", False)
         self.set_default("general.disable_notification_sound", False)
         self.set_default("general.scaling", "default")
+        self.set_default("general.font_scaling_mode", "default")
         self.set_default("run.working_directory", os.path.expanduser("~"))
 
     def _get_logging_level(self) -> int:
@@ -338,17 +341,23 @@ class Workbench(tk.Tk):
 
     def _load_plugins_from_path(self, path: List[str], prefix: str) -> None:
         load_function_name = "load_plugin"
-
+        
+        modules = []
         for _, module_name, _ in sorted(
             pkgutil.iter_modules(path, prefix), key=lambda x: x[2]
         ):
             try:
                 m = importlib.import_module(module_name)
                 if hasattr(m, load_function_name):
-                    getattr(m, load_function_name)()
+                    modules.append(m)
             except Exception:
                 logging.exception("Failed loading plugin '" + module_name + "'")
 
+        def module_sort_key(m):
+            return getattr(m, "load_order_key", m.__name__)
+        
+        for m in sorted(modules, key=module_sort_key):
+            getattr(m, load_function_name)()
     def _init_fonts(self) -> None:
         # set up editor and shell fonts
         self.set_default(
@@ -688,6 +697,7 @@ class Workbench(tk.Tk):
         include_in_toolbar: bool = False,
         submenu: Optional[tk.Menu] = None,
         bell_when_denied: bool = True,
+        show_extra_sequences = False
     ) -> None:
         """Registers an item to be shown in specified menu.
         
@@ -732,6 +742,7 @@ class Workbench(tk.Tk):
                 include_in_toolbar=include_in_toolbar,
                 submenu=submenu,
                 bell_when_denied=bell_when_denied,
+                show_extra_sequences=show_extra_sequences,
             )
         )
 
@@ -758,6 +769,7 @@ class Workbench(tk.Tk):
         include_in_toolbar: bool = False,
         submenu: Optional[tk.Menu] = None,
         bell_when_denied: bool = True,
+        show_extra_sequences: bool = False,
     ) -> None:
         def dispatch(event=None):
             if not tester or tester():
@@ -821,7 +833,13 @@ class Workbench(tk.Tk):
 
         if not accelerator and sequence:
             accelerator = sequence_to_accelerator(sequence)
-
+            """
+            # Does not work on Mac
+            if show_extra_sequences:
+                for extra_seq in extra_sequences:
+                    accelerator += " or " + sequence_to_accelerator(extra_seq)
+            """
+            
         # remember the details that can't be stored in Tkinter objects
         self._menu_item_specs[(menu_name, command_label)] = MenuItem(
             group, position_in_group, tester
@@ -1489,39 +1507,61 @@ class Workbench(tk.Tk):
 
     def _init_scaling(self) -> None:
         self._default_scaling_factor = self.tk.call("tk", "scaling")
+        if self._default_scaling_factor > 10:
+            # it may be infinity in eg. Fedora
+            self._default_scaling_factor = 1.33
 
         scaling = self.get_option("general.scaling")
         if scaling in ["default", "auto"]:  # auto was used in 2.2b3
             self._scaling_factor = self._default_scaling_factor
         else:
             self._scaling_factor = float(scaling)
-
+        
+        MAC_SCALING_MODIFIER = 1.7
         if running_on_mac_os():
-            self._scaling_factor *= 1.7
+            self._scaling_factor *= MAC_SCALING_MODIFIER
 
         self.tk.call("tk", "scaling", self._scaling_factor)
-
-        if running_on_linux() and scaling not in ["default", "auto"]:
+        
+        font_scaling_mode = self.get_option("general.font_scaling_mode")
+        
+        if (running_on_linux() 
+            and font_scaling_mode in ["default", "extra"]
+            and scaling not in ["default", "auto"]):
             # update system fonts which are given in pixel sizes
             for name in tk_font.names():
                 f = tk_font.nametofont(name)
                 orig_size = f.cget("size")
+                # According to do documentation, absolute values of negative font sizes 
+                # should be interpreted as pixel sizes (not affected by "tk scaling")
+                # and positive values are point sizes, which are supposed to scale automatically
+                # http://www.tcl.tk/man/tcl8.6/TkCmd/font.htm#M26
+                
+                # Unfortunately it seems that this cannot be relied on
+                # https://groups.google.com/forum/#!msg/comp.lang.tcl/ZpL6tq77M4M/GXImiV2INRQJ
+                
+                # My experiments show that manually changing negative font sizes 
+                # doesn't have any effect -- fonts keep their default size
+                # (Tested in Raspbian Stretch, Ubuntu 18.04 and Fedora 29)
+                # On the other hand positive sizes scale well (and they don't scale automatically)
+                
+                # convert pixel sizes to point_size
                 if orig_size < 0:
-                    # meaning its absolute value means height in pixels
-                    f.configure(
-                        size=int(
-                            orig_size
-                            * (self._scaling_factor / self._default_scaling_factor)
-                        )
-                    )
+                    orig_size = -orig_size / self._default_scaling_factor
+                
+                # scale
+                scaled_size = round(orig_size
+                        * (self._scaling_factor / self._default_scaling_factor)) 
+                f.configure(size=scaled_size)
+                
         elif running_on_mac_os() and scaling not in ["default", "auto"]:
-            # TODO: see http://wiki.tcl.tk/44444
+            # see http://wiki.tcl.tk/44444
             # update system fonts
             for name in tk_font.names():
                 f = tk_font.nametofont(name)
                 orig_size = f.cget("size")
                 assert orig_size > 0
-                f.configure(size=int(orig_size * self._scaling_factor / 1.7))
+                f.configure(size=int(orig_size * self._scaling_factor / MAC_SCALING_MODIFIER))
 
     def update_fonts(self) -> None:
         editor_font_size = self._guard_font_size(
@@ -2073,6 +2113,9 @@ class Workbench(tk.Tk):
 
         # Fallback
         webbrowser.open(url, False, True)
+    
+    def open_help_topic(self, topic, fragment=None):
+        self.show_view("HelpView").load_topic(topic, fragment)
 
     def bell(self, displayof=0):
         if not self.get_option("general.disable_notification_sound"):
@@ -2080,6 +2123,9 @@ class Workbench(tk.Tk):
 
     def _mac_quit(self, *args):
         self._on_close()
+    
+    def get_toolbar(self):
+        return self._toolbar
 
 
 class WorkbenchEvent(Record):
